==> rl-3.1.tar.gz: rl-3.1/rl/unicode.c <==
#include "Python.h"
#include "unicode.h"

#if (PY_MAJOR_VERSION >= 3)

/* PyMem_RawMalloc appeared in Python 3.4 */
#if (PY_VERSION_HEX < 0x03040000)
#define PyMem_RawMalloc PyMem_Malloc
#define PyMem_RawFree PyMem_Free
#endif

/* Always use surrogateescape error handler */
#define _ERRORS "surrogateescape"


/* Unicode support */

PyObject *
PyUnicode_DECODE(const char *text)
{
	return PyUnicode_DecodeLocale(text, _ERRORS);
}


PyObject *
PyUnicode_ENCODE(PyObject *text)
{
	return PyUnicode_EncodeLocale(text, _ERRORS);
}


PyObject *
PyUnicode_FS_DECODE(const char *text)
{
	return PyUnicode_DecodeFSDefault(text);
}


PyObject *
PyUnicode_FS_ENCODE(PyObject *text)
{
	return PyUnicode_EncodeFSDefault(text);
}


PyObject *
PyUnicode_DECODE_CHAR(char character)
{
	char text[2] = "\0";

	text[0] = character;
	return PyUnicode_DECODE(text);
}


Py_ssize_t
PyUnicode_INDEX(const char *text, Py_ssize_t index)
{
	PyObject *u;
	Py_ssize_t i;

	/* Cast away const */
	char *s = (char*)text;
	char saved;
	size_t l;

	/* Short-circuit */
	if (index == 0)
		return 0;

	l = strlen(text);
	if (index > l)
		index = l;
	saved = s[index];
	s[index] = '\0';
	u = PyUnicode_DECODE(s);
	s[index] = saved;
	if (u == NULL)
		return -1;
	i = PyUnicode_GET_LENGTH(u);
	Py_DECREF(u);
	return i;
}


int
PyUnicode_StrConverter(PyObject *text, void *addr)
{
	PyObject *b;

	/* Cleanup stage */
	if (text == NULL) {
		Py_DECREF(*(PyObject**)addr);
		return 1;
	}
	/* Conversion stage */
	b = PyUnicode_ENCODE(text);
	if (b == NULL)
		return 0;
	if (PyBytes_GET_SIZE(b) != strlen(PyBytes_AS_STRING(b))) {
		PyErr_SetString(PyExc_TypeError, "embedded NUL character");
		Py_DECREF(b);
		return 0;
	}
	*(PyObject**)addr = b;
	return Py_CLEANUP_SUPPORTED;
}


int
PyUnicode_FSOrNoneConverter(PyObject *text, void *addr)
{
	PyObject *b;

	/* Cleanup stage */
	if (text == NULL) {
		Py_DECREF(*(PyObject**)addr);
		return 1;
	}
	/* Conversion stage */
	if (text == Py_None) {
		*(PyObject**)addr = NULL;
		return 1;
	}
#if (PY_VERSION_HEX >= 0x03060000)
	{
		PyObject *path = PyOS_FSPath(text);
		if (path == NULL)
			return 0;
		if (PyBytes_Check(path))
			b = path;
		else {
			b = PyUnicode_FS_ENCODE(path);
			Py_DECREF(path);
			if (b == NULL)
				return 0;
		}
	}
#else
	if (PyBytes_Check(text)) {
		b = text;
		Py_INCREF(b);
	}
	else {
		b = PyUnicode_FS_ENCODE(text);
		if (b == NULL)
			return 0;
	}
#endif
	if (PyBytes_GET_SIZE(b) != strlen(PyBytes_AS_STRING(b))) {
		PyErr_SetString(PyExc_TypeError, "embedded NUL character");
		Py_DECREF(b);
		return 0;
	}
	*(PyObject**)addr = b;
	return Py_CLEANUP_SUPPORTED;
}

#endif /* Python 3 */


PyObject *
PyUnicode_GetPreferredEncoding()
{
	PyObject *locale = NULL;
	PyObject *getpreferredencoding = NULL;
	PyObject *r = NULL;

	locale = PyImport_ImportModule("locale");
	if (locale == NULL)
		goto error;

	getpreferredencoding = PyObject_GetAttrString(locale, "getpreferredencoding");
	if (getpreferredencoding == NULL)
		goto error;

	r = PyObject_CallFunction(getpreferredencoding, "i", 0);
	if (r == NULL)
		goto error;

	Py_DECREF(locale);
	Py_DECREF(getpreferredencoding);
	return r;
  error:
	Py_XDECREF(locale);
	Py_XDECREF(getpreferredencoding);
	return NULL;
}


int
PyUnicode_CopyPreferredEncoding(char *buffer, Py_ssize_t buffer_size)
{
	PyObject *u = NULL;
	PyObject *b = NULL;
	Py_ssize_t len;

#if (PY_MAJOR_VERSION >= 3)
	u = PyUnicode_GetPreferredEncoding();
	if (u == NULL)
		goto error;
	b = PyUnicode_AsASCIIString(u);
#else
	b = PyUnicode_GetPreferredEncoding();
#endif
	if (b == NULL)
		goto error;

	len = PyBytes_GET_SIZE(b);
	if (len >= buffer_size)
		len = buffer_size - 1;

	strncpy(buffer, PyBytes_AS_STRING(b), len);
	buffer[len] = '\0';

	Py_XDECREF(u);
	Py_DECREF(b);
	return 1;
  error:
	Py_XDECREF(u);
	Py_XDECREF(b);
	return 0;
}


void
PyUnicode_PrintEncodings()
{
	char buffer[128] = "";
	PyUnicode_CopyPreferredEncoding(buffer, 128);

	printf("%s %s\n", buffer, Py_FileSystemDefaultEncoding);
}

==> rl-3.2.tar.gz: rl-3.2/rl/unicode.c <==
#include "Python.h"
#include "unicode.h"

#if (PY_MAJOR_VERSION >= 3)

/* PyMem_RawMalloc appeared in Python 3.4 */
#if (PY_VERSION_HEX < 0x03040000)
#define PyMem_RawMalloc PyMem_Malloc
#define PyMem_RawFree PyMem_Free
#endif

/* Always use surrogateescape error handler */
#define _ERRORS "surrogateescape"


/* Unicode support */

PyObject *
PyUnicode_DECODE(const char *text)
{
	return PyUnicode_DecodeLocale(text, _ERRORS);
}


PyObject *
PyUnicode_ENCODE(PyObject *text)
{
	return PyUnicode_EncodeLocale(text, _ERRORS);
}


PyObject *
PyUnicode_FS_DECODE(const char *text)
{
	return PyUnicode_DecodeFSDefault(text);
}


PyObject *
PyUnicode_FS_ENCODE(PyObject *text)
{
	return PyUnicode_EncodeFSDefault(text);
}


PyObject *
PyUnicode_DECODE_CHAR(char character)
{
	char text[2] = "\0";

	text[0] = character;
	return PyUnicode_DECODE(text);
}


Py_ssize_t
PyUnicode_INDEX(const char *text, Py_ssize_t index)
{
	PyObject *u;
	Py_ssize_t i;

	/* Cast away const */
	char *s = (char*)text;
	char saved;
	Py_ssize_t l;

	/* Short-circuit */
	if (index == 0)
		return 0;

	l = strlen(text);
	if (index > l)
		index = l;
	saved = s[index];
	s[index] = '\0';
	u = PyUnicode_DECODE(s);
	s[index] = saved;
	if (u == NULL)
		return -1;
	i = PyUnicode_GET_LENGTH(u);
	Py_DECREF(u);
	return i;
}


int
PyUnicode_StrConverter(PyObject *text, void *addr)
{
	PyObject *b;

	/* Cleanup stage */
	if (text == NULL) {
		Py_DECREF(*(PyObject**)addr);
		return 1;
	}
	/* Conversion stage */
	b = PyUnicode_ENCODE(text);
	if (b == NULL)
		return 0;
	if (PyBytes_GET_SIZE(b) != (Py_ssize_t)strlen(PyBytes_AS_STRING(b))) {
		PyErr_SetString(PyExc_TypeError, "embedded NUL character");
		Py_DECREF(b);
		return 0;
	}
	*(PyObject**)addr = b;
	return Py_CLEANUP_SUPPORTED;
}


int
PyUnicode_FSOrNoneConverter(PyObject *text, void *addr)
{
	PyObject *b;

	/* Cleanup stage */
	if (text == NULL) {
		Py_DECREF(*(PyObject**)addr);
		return 1;
	}
	/* Conversion stage */
	if (text == Py_None) {
		*(PyObject**)addr = NULL;
		return 1;
	}
#if (PY_VERSION_HEX >= 0x03060000)
	{
		PyObject *path = PyOS_FSPath(text);
		if (path == NULL)
			return 0;
		if (PyBytes_Check(path))
			b = path;
		else {
			b = PyUnicode_FS_ENCODE(path);
			Py_DECREF(path);
			if (b == NULL)
				return 0;
		}
	}
#else
	if (PyBytes_Check(text)) {
		b = text;
		Py_INCREF(b);
	}
	else {
		b = PyUnicode_FS_ENCODE(text);
		if (b == NULL)
			return 0;
	}
#endif
	if (PyBytes_GET_SIZE(b) != (Py_ssize_t)strlen(PyBytes_AS_STRING(b))) {
		PyErr_SetString(PyExc_TypeError, "embedded NUL character");
		Py_DECREF(b);
		return 0;
	}
	*(PyObject**)addr = b;
	return Py_CLEANUP_SUPPORTED;
}

#endif /* Python 3 */


PyObject *
PyUnicode_GetPreferredEncoding()
{
	PyObject *locale = NULL;
	PyObject *getpreferredencoding = NULL;
	PyObject *r = NULL;

	locale = PyImport_ImportModule("locale");
	if (locale == NULL)
		goto error;

	getpreferredencoding = PyObject_GetAttrString(locale, "getpreferredencoding");
	if (getpreferredencoding == NULL)
		goto error;

	r = PyObject_CallFunction(getpreferredencoding, "i", 0);
	if (r == NULL)
		goto error;

	Py_DECREF(locale);
	Py_DECREF(getpreferredencoding);
	return r;
  error:
	Py_XDECREF(locale);
	Py_XDECREF(getpreferredencoding);
	return NULL;
}


PyObject *
PyUnicode_GetFileSystemEncoding()
{
	PyObject *sys = NULL;
	PyObject *getfilesystemencoding = NULL;
	PyObject *r = NULL;

	sys = PyImport_ImportModule("sys");
	if (sys == NULL)
		goto error;

	getfilesystemencoding = PyObject_GetAttrString(sys, "getfilesystemencoding");
	if (getfilesystemencoding == NULL)
		goto error;

	r = PyObject_CallFunction(getfilesystemencoding, NULL);
	if (r == NULL)
		goto error;

	Py_DECREF(sys);
	Py_DECREF(getfilesystemencoding);
	return r;
  error:
	Py_XDECREF(sys);
	Py_XDECREF(getfilesystemencoding);
	return NULL;
}


int
PyUnicode_CopyAsString(PyObject *text, char *buffer, Py_ssize_t buffer_size)
{
	PyObject *b = NULL;
	Py_ssize_t len;

	if (text == NULL) {
		goto error;
	}
	if (text == Py_None) {
		goto error;
	}
	if (PyBytes_Check(text)) {
		b = text;
		Py_INCREF(b);
	}
	else {
		b = PyUnicode_AsASCIIString(text);
		if (b == NULL)
			goto error;
	}

	len = PyBytes_GET_SIZE(b);
	if (len >= buffer_size)
		len = buffer_size - 1;

	strncpy(buffer, PyBytes_AS_STRING(b), len);
	buffer[len] = '\0';

	Py_DECREF(b);
	return 1;
  error:
	Py_XDECREF(b);
	return 0;
}


void
PyUnicode_PrintEncodings()
{
	char preferred[128] = "";
	char filesystem[128] = "";

	PyUnicode_CopyAsString(PyUnicode_GetPreferredEncoding(), preferred, 128);
	PyUnicode_CopyAsString(PyUnicode_GetFileSystemEncoding(), filesystem, 128);

	printf("%s %s\n", preferred, filesystem);
}
